@@ -134,6 +134,47 @@ namespace GEO {
 
     /************************************************************************/
 
+    /**
+     * \brief Computes the determinant of a 2x2 matrix
+     * \param[in] M a const reference to the matrix
+     * \return the determinant
+     */
+    inline double det(const mat2& M) {
+        return det2x2(
+            M(0,0), M(0,1),
+            M(1,0), M(1,1)
+        );
+    }
+
+    /**
+     * \brief Computes the determinant of a 3x3 matrix
+     * \param[in] M a const reference to the matrix
+     * \return the determinant
+     */
+    inline double det(const mat3& M) {
+        return det3x3(
+            M(0,0), M(0,1), M(0,2),
+            M(1,0), M(1,1), M(1,2),
+            M(2,0), M(2,1), M(2,2)
+        );
+    }
+
+    /**
+     * \brief Computes the determinant of a 4x4 matrix
+     * \param[in] M a const reference to the matrix
+     * \return the determinant
+     */
+    inline double det(const mat4& M) {
+        return det4x4(
+            M(0,0), M(0,1), M(0,2), M(0,3),
+            M(1,0), M(1,1), M(1,2), M(1,3),
+            M(2,0), M(2,1), M(2,2), M(2,3),
+            M(3,0), M(3,1), M(3,2), M(3,3)
+        );
+    }
+    
+    /************************************************************************/    
+
     /**
      * \brief Geometric functions and utilities.
      */
@@ -277,18 +318,6 @@ namespace GEO {
             return cross(p2 - p1, p3 - p1);
         }
 
-        /**
-         * \brief Gets the axis that is most normal to a triangle
-         * \param[in] p1 , p2 , p3 the three vertices of the 
-         *    triangle
-         * \return the coordinate of the normal vector with the 
-         *    greatest absolute value
-         */
-        coord_index_t GEOGRAM_API triangle_normal_axis(
-            const vec3& p1, const vec3& p2, const vec3& p3
-        );
-
-        
         /**
          * \brief Computes the area of a 3d triangle
          * \param[in] p1 , p2 , p3 the three vertices of the triangle
@@ -702,8 +731,8 @@ namespace GEO {
      */
     class Box2d {
     public:
-        double xy_min[3];
-        double xy_max[3];
+        double xy_min[2];
+        double xy_max[2];
 
         /**
          * \brief Tests whether a box contains a point.
@@ -832,6 +861,46 @@ namespace GEO {
         ) ; 
     }
 
+
+    /**
+     * \brief Applies a 3d transform to a 3d point.
+     * \details Convention is the same as in math, i.e.
+     *  vector is a column vector, multiplied on the right
+     *  of the transform.
+     *  Internally, the point is converted into
+     *  a 4d vector, with w coordinate set to one. Transformed
+     *  coordinates are divided by the transformed w to form
+     *  a 3d point.
+     * \param[in] v the input 3d point to be transformed
+     * \param[in] m the transform, as a 4x4 matrix, using
+     *  homogeneous coordinates
+     * \tparam FT type of the coordinates
+     * \return the transformed 3d point
+     */
+    template <class FT> vecng<3,FT> transform_point(
+        const Matrix<4,FT>& m,        
+        const vecng<3,FT>& v
+    ){
+        index_t i,j ;
+        FT result[4] ;
+        
+        for(i=0; i<4; i++) {
+            result[i] = 0 ;
+        }
+        for(i=0; i<4; i++) {
+            for(j=0; j<3; j++) {
+                result[i] += v[j] * m(i,j) ;
+            }
+            result[i] += m(i,3);
+        }
+    
+        return vecng<3,FT>(
+            result[0] / result[3],
+            result[1] / result[3],
+            result[2] / result[3] 
+        ) ; 
+    }
+    
     /**
      * \brief Applies a 4d transform to a 4d point.
      * \details Convention is the same as in OpenGL, i.e.
