@@ -42,8 +42,10 @@
 
 #include <geogram/basic/common.h>
 #include <geogram/basic/numeric.h>
+#include <geogram/basic/determinant.h>
 #include <geogram/basic/memory.h>
 #include <geogram/basic/assert.h>
+#include <initializer_list>
 
 #include <iostream>
 #include <cfloat>
@@ -131,6 +133,19 @@ namespace GEO {
             }
         }
 
+        /** 
+         * \brief Constructs a vector from an initializer list.
+         * \param[in] Vi the initializer list, should contain DIM elements.
+         */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < DIM);
+                data()[i] = it;
+                ++i;
+            }
+        }
+        
         /**
          * \brief Gets the vector dimension
          * \return the value of \p DIM
@@ -542,6 +557,16 @@ namespace GEO {
             y(v[1]) {
         }
 
+        /** \copydoc vecng::vecng(const std::initializer_list<T>) */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < dim);
+                data()[i] = it;
+                ++i;
+            }
+        }
+        
         /** \copydoc vecng::length2() const */
         inline T length2() const {
             return x * x + y * y;
@@ -648,6 +673,12 @@ namespace GEO {
             return data()[i];
         }
 
+        /** \brief Optimizes coordinate representation */
+        void optimize() {
+            Numeric::optimize_number_representation(x);
+            Numeric::optimize_number_representation(y);
+        }
+        
         /** \brief Vector x coordinate */
         T x;
         /** \brief Vector y coordinate */
@@ -710,9 +741,9 @@ namespace GEO {
 
         /** \copydoc vecng::vecng() */
         vecng() :
-            x(0),
-            y(0),
-            z(0) {
+            x(T(0.0)),
+            y(T(0.0)),
+            z(T(0.0)) {
         }
 
         /**
@@ -741,6 +772,16 @@ namespace GEO {
             z(v[2]) {
         }
 
+        /** \copydoc vecng::vecng(const std::initializer_list<T>) */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < dim);
+                data()[i] = it;
+                ++i;
+            }
+        }
+        
         /** \copydoc vecng::length2() const */
         inline T length2() const {
             return x * x + y * y + z * z;
@@ -852,6 +893,13 @@ namespace GEO {
             return data()[i];
         }
 
+        /** \brief Optimizes coordinate representation */
+        void optimize() {
+            Numeric::optimize_number_representation(x);
+            Numeric::optimize_number_representation(y);
+            Numeric::optimize_number_representation(z);
+        }
+        
         /** \brief Vector x coordinate */
         T x;
         /** \brief Vector y coordinate */
@@ -883,9 +931,9 @@ namespace GEO {
         const vecng<3, T>& v1, const vecng<3, T>& v2
     ) {
         return vecng<3, T>(
-            v1.y * v2.z - v1.z * v2.y,
-            v1.z * v2.x - v1.x * v2.z,
-            v1.x * v2.y - v1.y * v2.x
+            det2x2(v1.y, v2.y, v1.z, v2.z),
+            det2x2(v1.z, v2.z, v1.x, v2.x),
+            det2x2(v1.x, v2.x, v1.y, v2.y)
         );
     }
 
@@ -955,6 +1003,16 @@ namespace GEO {
             w(v[3]) {
         }
 
+        /** \copydoc vecng::vecng(const std::initializer_list<T>) */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < dim);
+                data()[i] = it;
+                ++i;
+            }
+        }
+        
         /** \copydoc vecng::length2() const */
         inline T length2() const {
             return x * x + y * y + z * z + w * w;
@@ -1142,6 +1200,29 @@ namespace GEO {
         }
         return in;
     }
+
+    /************************************************************************/
+
+    namespace Numeric {
+        
+        template<class T> 
+        inline void optimize_number_representation(
+            vecng<2,T>& v
+        ) {
+            v.optimize();
+        }
+
+        template<class T>
+        inline void optimize_number_representation(
+            vecng<3,T>& v
+        ) {
+            v.optimize();
+        }
+        
+    }
+
+    /************************************************************************/
+    
 }
 
 #endif
