@@ -43,15 +43,13 @@
 #include <geogram/basic/common.h>
 #include <geogram/basic/numeric.h>
 #include <geogram/basic/geometry.h>
+#include <geogram/numerics/PCK.h>
 
 /**
  * \file geogram/numerics/predicates.h
  * \brief Filtered exact predicates for restricted Voronoi diagrams.
  */
 
-// Uncomment to get full reporting on predicate statistics
-// (but has a non-negligible impact on performance)
-// #define PCK_STATS
 
 namespace GEO {
 
@@ -184,8 +182,8 @@ namespace GEO {
          *  and \p p3
          * \param[in] q0 , q1 , q2 vertices of the triangle
          *  (that defines the intersection q)
-	 * \param[in] SOS if true, do the symbolic perturbation in the degenerate
-	 *  case
+	 * \param[in] SOS if true, do the symbolic perturbation in the 
+         *  degenerate case
          * \retval POSITIVE if d(p0 hp0,q) < d(p3 hp3, q)
          * \retval NEGATIVE if d(p0 hp0,q) > d(p3 hp3, q)
          * \retval perturb() if d(p0 hp0,q) = d(p3 hp3, q),
@@ -300,11 +298,8 @@ namespace GEO {
         /**
          * \brief Tests whether a 3d point is inside the circumscribed 
          *  sphere of a 3d tetrahedron.
-         * \param[in] p0 first vertex of the tetrahedron
-         * \param[in] p1 second vertex of the tetrahedron
-         * \param[in] p2 third vertex of the tetrahedron
-         * \param[in] p3 fourth vertex of the tetrahedron
-         * \param[in] p4 the point to be tested
+         * \param[in] p0 , p1 , p2 , p3 the four vertices of the tetrahedron
+         * \param[in] p4 the point
          * \retval POSITIVE whenever \p p4 is inside the circumscribed sphere
          *  of the tetrahedron \p p0, \p p1, \p p2, \p p3
          * \retval NEGATIVE whenever \p p4 is outside the circumscribed sphere
@@ -395,15 +390,13 @@ namespace GEO {
         );
         
         /**
-         * \brief Computes the orientation predicate in 3d.
+         * \brief Computes the orientation predicate in 2d.
          * \details Computes the sign of the signed area of
          *  the triangle p0, p1, p2.
          * \param[in] p0 , p1 , p2 vertices of the triangle
-         * \retval POSITIVE if the triangle is oriented positively
+         * \retval POSITIVE if the triangle is oriented counter-clockwise
          * \retval ZERO if the triangle is flat
-         * \retval NEGATIVE if the triangle is oriented negatively
-         * \todo check whether orientation is inverted as compared to 
-         *   Shewchuk's version.
+         * \retval NEGATIVE if the triangle is oriented clockwise
          */
         Sign GEOGRAM_API orient_2d(
             const double* p0, const double* p1, const double* p2
@@ -416,11 +409,9 @@ namespace GEO {
          * \details Computes the sign of the signed area of
          *  the triangle p0, p1, p2.
          * \param[in] p0 , p1 , p2 vertices of the triangle
-         * \retval POSITIVE if the triangle is oriented positively
+         * \retval POSITIVE if the triangle is oriented counter-clockwise
          * \retval ZERO if the triangle is flat
-         * \retval NEGATIVE if the triangle is oriented negatively
-         * \todo check whether orientation is inverted as compared to 
-         *   Shewchuk's version.
+         * \retval NEGATIVE if the triangle is oriented clockwise
          */
         inline Sign orient_2d(
             const vec2& p0, const vec2& p1, const vec2& p2
@@ -432,7 +423,7 @@ namespace GEO {
         /**
          * \brief Computes the 3d orientation test with lifted points.
          * \details Given three lifted points p0', p1', p2' in 
-         *  R^3, tests if the lifted point p3' in R^3 lies below or above 
+         *  R^2, tests if the lifted point p3' in R^3 lies below or above 
          *  the plane passing through the three points 
          *  p0', p1', p2'.
          *  The first two coordinates and the
@@ -462,8 +453,6 @@ namespace GEO {
          * \retval POSITIVE if the tetrahedron is oriented positively
          * \retval ZERO if the tetrahedron is flat
          * \retval NEGATIVE if the tetrahedron is oriented negatively
-         * \todo check whether orientation is inverted as compared to 
-         *   Shewchuk's version.
          */
         Sign GEOGRAM_API orient_3d(
             const double* p0, const double* p1,
@@ -480,8 +469,6 @@ namespace GEO {
          * \retval POSITIVE if the tetrahedron is oriented positively
          * \retval ZERO if the tetrahedron is flat
          * \retval NEGATIVE if the tetrahedron is oriented negatively
-         * \todo check whether orientation is inverted as compared to 
-         *   Shewchuk's version.
          */
         inline Sign orient_3d(
             const vec3& p0, const vec3& p1,
@@ -689,8 +676,6 @@ namespace GEO {
 	 * \retval POSITIVE if the tetrahedron is oriented positively
 	 * \retval ZERO if the tetrahedron is flat
 	 * \retval NEGATIVE if the tetrahedron is oriented negatively
-	 * \todo check whether orientation is inverted as compared to 
-	 *   Shewchuk's version.
 	 */
 	inline Sign orient_3d_inexact(
 	    const double* p0, const double* p1,
