@@ -55,6 +55,17 @@
 typedef double GLdouble;
 #endif
 
+/**
+ * \brief Used internally
+ * \details GLUP_THICK_LINES is the primitive used to draw GLUP_LINES
+ *  when mesh width is greater than 1. It is a different primitive because
+ *  it needs either a geometry shader to replace line segments with quads
+ *  (GLUPGLSL 150 and 440 profiles), or a pre-processing of the immediate 
+ *  vertex buffers to generate two additional vertices per segment with 
+ *  the attributes (in GLUPES profile, WIP, to be implemented).
+ */
+static constexpr GLUPprimitive GLUP_THICK_LINES = GLUP_RESERVED_PRIMITIVE_1;
+
 namespace GLUP {
     using namespace GEO;
 
@@ -166,11 +177,23 @@ namespace GLUP {
      */
     void show_matrix(const GLfloat m[16]);
 
+    /**
+     * \brief For debugging, outputs a matrix to the standard error.
+     * \param[in] m the matrix to be displayed.
+     */
+    void show_matrix(const GLdouble m[16]);
+    
     /**
      * \brief For debugging, outputs a vector to the standard error.
      * \param[in] v the vector to be displayed
      */
     void show_vector(const GLfloat v[4]);
+
+    /**
+     * \brief For debugging, outputs a vector to the standard error.
+     * \param[in] v the vector to be displayed
+     */
+    void show_vector(const GLdouble v[4]);
     
     /**
      * \brief Resets a matrix to the identity matrix.
@@ -178,6 +201,12 @@ namespace GLUP {
      */
     void load_identity_matrix(GLfloat out[16]);
 
+    /**
+     * \brief Resets a matrix to the identity matrix.
+     * \param[out] out the matrix to be reset.
+     */
+    void load_identity_matrix(GLdouble out[16]);
+    
     /**
      * \brief Copies a vector of floats.
      * \param[out] to a pointer to the destination vector
@@ -188,6 +217,16 @@ namespace GLUP {
         Memory::copy(to, from, sizeof(GLfloat)*dim);
     }
 
+    /**
+     * \brief Copies a vector of doubles.
+     * \param[out] to a pointer to the destination vector
+     * \param[in] from a const pointer to the source vector
+     * \param[in] dim the number of components to copy
+     */
+    inline void copy_vector(GLdouble* to, const GLdouble* from, index_t dim) {
+        Memory::copy(to, from, sizeof(GLdouble)*dim);
+    }
+    
     /**
      * \brief Copies a vector of doubles to a vector of floats.
      * \param[out] to a pointer to the destination vector
@@ -224,14 +263,6 @@ namespace GLUP {
         v[2] *= s;            
     }
 
-
-    /**
-     * \brief Gives the number of vertices for each GLUP primitive.
-     * \details The array is indexed by the GLUP primitive type
-     *  (one of GLUP_POINTS, GLUP_LINES, ...)
-     */
-    extern index_t nb_vertices_per_primitive[];
-    
     /**********************************************************************/
     
     class Context;
@@ -263,7 +294,7 @@ namespace GLUP {
          * \return a pointer to the coefficients of
          *  the matrix.
          */
-        GLfloat* top() {
+        GLdouble* top() {
             return stack_[top_].data();
         }
 
@@ -272,9 +303,9 @@ namespace GLUP {
          */
         void push() {
             geo_assert(top_ != MAX_DEPTH-1);
-            GLfloat* from = top();
+            GLdouble* from = top();
             ++top_;
-            GLfloat* to = top();
+            GLdouble* to = top();
             copy_vector(to, from, 16);
         }
 
@@ -289,8 +320,8 @@ namespace GLUP {
 
     protected:
         struct Matrix {
-            GLfloat coeff[16];
-            GLfloat* data() {
+            GLdouble coeff[16];
+            GLdouble* data() {
                 return &coeff[0];
             }
         };
@@ -418,6 +449,22 @@ namespace GLUP {
             }
         }
 
+        /**
+         * \brief Copies this attribute from another attribute
+         * \param[in] to index of the destination vertex
+         * \param[in] from_buffer the source buffer
+         * \param[in] from index of the source vertex
+         * \pre to < IMMEDIATE_BUFFER_SIZE && from < IMMEDIATE_BUFFER_SIZE
+         */
+        void copy(index_t to, ImmediateBuffer& from_buffer, index_t from) {
+            geo_debug_assert(to < IMMEDIATE_BUFFER_SIZE);
+            geo_debug_assert(from < IMMEDIATE_BUFFER_SIZE);
+            geo_debug_assert(from_buffer.dimension() == dimension());
+            copy_vector(
+                element_ptr(to), from_buffer.element_ptr(from), dimension()
+            );
+        }
+        
         /**
          * \brief Gets the dimension of the attribute.
          * \return the number of components of the attribute
@@ -499,12 +546,16 @@ namespace GLUP {
     public:
         /**
          * \brief ImmediateState constructor.
+         * \param[in] nb_vertices_per_primitive a pointer to an array of index_t
+         *  of size GLUP_NB_PRIMITIVES that indicates for each primitive the
+         *  number of vertices (3 for GLUP_TRIANGLES, 4 for GLUP_QUADS etc...).
          */
-        ImmediateState() :
+        ImmediateState(index_t* nb_vertices_per_primitive) :
             current_vertex_(0),
             max_current_vertex_(0),
             primitive_(GLUP_POINTS),
-            VAO_(0)
+            VAO_(0),
+            nb_vertices_per_primitive_(nb_vertices_per_primitive)
 	{
             buffer[GLUP_VERTEX_ATTRIBUTE].initialize(4);
             buffer[GLUP_COLOR_ATTRIBUTE].initialize(4);
@@ -552,14 +603,23 @@ namespace GLUP {
          * \brief Configures the immediate state for rendering
          *  primitives of a given type.
          * \param[in] primitive type of the primitives to be rendered
+         * \param[in] max_current_vertex optional maximum index of a vertex
+         *   index in immediate buffer before flushing. Computed from primitive
+         *   if unspecified. It is used by primitives that need additional
+         *   vertices to be generated in the immediate buffer (e.g., 
+         *   GLUP_THICK_LINES in GLUP_ES profile).
          */
-        void begin(GLUPprimitive primitive) {
+        void begin(GLUPprimitive primitive, index_t max_current_vertex=0) {
             current_vertex_ = 0;
-            max_current_vertex_ =
-            IMMEDIATE_BUFFER_SIZE - (
-                IMMEDIATE_BUFFER_SIZE %
-                nb_vertices_per_primitive[primitive]
-            );
+            if(max_current_vertex != 0) {
+                max_current_vertex_ = max_current_vertex;
+            } else {
+                max_current_vertex_ =
+                    IMMEDIATE_BUFFER_SIZE - (
+                        IMMEDIATE_BUFFER_SIZE %
+                        nb_vertices_per_primitive_[primitive]
+                    );
+            }
             primitive_ = primitive;
         }
 
@@ -577,7 +637,7 @@ namespace GLUP {
 
         /**
          * \brief Tests whether the buffers are full.
-         * \details When buffers are fulled, their contents need to be sent
+         * \details When buffers are full, their contents need to be sent
          *  to OpenGL before calling reset(). These operations are done
          *  by the Context.
          */
@@ -586,10 +646,12 @@ namespace GLUP {
         }
 
         /**
-         * \brief Resets the current vertex index to zero.
+         * \brief Resets the current vertex index.
+         * \param[in] new_current_vertex optional new value 
+         *   of the current vertex.
          */
-        void reset() {
-            current_vertex_ = 0;
+        void reset(index_t new_current_vertex = 0) {
+            current_vertex_ = new_current_vertex;
         }
 
         /**
@@ -611,7 +673,7 @@ namespace GLUP {
          * \brief Gets the number of primitives stored in the buffers.
          */
         index_t nb_primitives() const {
-            return current_vertex_ / nb_vertices_per_primitive[
+            return current_vertex_ / nb_vertices_per_primitive_[
                 primitive_
             ];
         }
@@ -645,6 +707,7 @@ namespace GLUP {
         index_t max_current_vertex_;
         GLUPprimitive primitive_;
         GLuint VAO_;
+        index_t* nb_vertices_per_primitive_;
     };
     
 
@@ -1092,6 +1155,17 @@ namespace GLUP {
             flag_matrices_as_dirty();
         }
 
+        /**
+         * \brief Replaces the top of the current matrix stack
+         *  with the specified matrix.
+         * \param[in] m the matrix that will replace the top of
+         *  the current matrix stack
+         */
+        void load_matrix(const GLdouble m[16]) {
+            copy_vector(matrix_stack_[matrix_mode_].top(), m, 16);
+            flag_matrices_as_dirty();
+        }
+        
         /**
          * \brief Replaces the top of the current matrix stack
          *  with the identity matrix.
@@ -1108,12 +1182,12 @@ namespace GLUP {
          *   top of the current matrix stack.
          * \see matrix_mode()
          */
-        void mult_matrix(const GLfloat m[16]) {
-            GLfloat product[16];
+        void mult_matrix(const GLdouble m[16]) {
+            GLdouble product[16];
             mult_matrices(product,m,matrix_stack_[matrix_mode_].top());
             load_matrix(product);
         }
-
+        
         /**
          * \brief Pushes a copy of the top of the current stack matrix
          *  onto the current stack matrix.
@@ -1317,7 +1391,7 @@ namespace GLUP {
          * \param[in] matrix name of the stack, one of GLUP_MODELVIEW_MATRIX,
          *   GLUP_PROJECTION_MATRIX, GLUP_TEXTURE_MATRIX
          */
-        GLUPfloat* get_matrix(GLUPmatrix matrix) {
+        GLUPdouble* get_matrix(GLUPmatrix matrix) {
             geo_debug_assert(matrix < 3);
             return matrix_stack_[matrix].top();
         }
@@ -1532,6 +1606,11 @@ namespace GLUP {
          */
         virtual void setup_GLUP_LINES();
 
+        /**
+         * \brief Setups GLSL programs for lines with width > 1.
+         */
+        virtual void setup_GLUP_THICK_LINES();
+        
         /**
          * \brief Setups GLSL programs for triangles.
          */
@@ -1951,6 +2030,16 @@ namespace GLUP {
          * \details It is used to emulate gl_VertexID in shaders.
          */
         GLuint vertex_id_VBO_;
+
+        /**
+         * \brief Number of vertices per primitive (3 for GLUP_TRIANGLES,
+         *   4 for GLUP_QUADS etc...)
+         * \details It is stored as a class member array rather than a 
+         *  static array so that particular implementations can change
+         *  it according to the needs (for instance, GLUPES profile temporarily
+         *  uses quads with for vertices to render GLUP_THICK_LINES).
+         */
+        index_t nb_vertices_per_primitive_[GLUP_NB_PRIMITIVES];
     };
 
     /*********************************************************************/
