@@ -127,10 +127,6 @@ namespace GEO {
          * \details The mesh visibility flags specifies
          *  whether mesh edges should be drawn. The used
          *  color can be specified by set_mesh_color()
-         * \note For now, regions are only implemented for
-         *   triangulated meshes and tetrahedralized meshes
-         *   (not implemented yet for hybrid surfacic and 
-         *    volumetric meshes).
          */
         void set_show_mesh(bool x) {
             show_mesh_ = x;
@@ -502,10 +498,7 @@ namespace GEO {
          *  MESH_VERTICES, MESH_EDGES, MESH_FACETS, MESH_CELLS,
          *  or MESH_NONE if picking mode should be deactivated
          * \note Picking mode is currently only implemented with
-         *  GLSL support at least v1.5, and only works for points,
-         *  segments, pure triangle meshes and pure tetrahedral meshes
-         *  (facet picking for polygonal meshes and cell picking for 
-         *   hybrid meshes are not implemented yet).
+         *  GLSL support at least v1.5.
          */
         void set_picking_mode(MeshElementsFlags what) {
             picking_mode_ = what;
@@ -640,6 +633,12 @@ namespace GEO {
          * \brief Unset primitive filtering
          */
         void unset_filters();
+
+
+        /**
+         * \brief Deallocates OpenGL objects
+         */
+        void cleanup();
         
     protected:
         
